@@ -41,10 +41,8 @@
 #include "gdm-display-access-file.h"
 #include "gdm-common.h"
 
-struct _GdmDisplayAccessFile
+struct _GdmDisplayAccessFilePrivate
 {
-        GObject parent;
-
         char *username;
         FILE *fp;
         char *path;
@@ -81,11 +79,11 @@ gdm_display_access_file_get_property (GObject    *object,
 
         switch (prop_id) {
             case PROP_USERNAME:
-                g_value_set_string (value, access_file->username);
+                g_value_set_string (value, access_file->priv->username);
                 break;
 
             case PROP_PATH:
-                g_value_set_string (value, access_file->path);
+                g_value_set_string (value, access_file->priv->path);
                 break;
 
             default:
@@ -105,8 +103,8 @@ gdm_display_access_file_set_property (GObject      *object,
 
         switch (prop_id) {
             case PROP_USERNAME:
-                g_assert (access_file->username == NULL);
-                access_file->username = g_value_dup_string (value);
+                g_assert (access_file->priv->username == NULL);
+                access_file->priv->username = g_value_dup_string (value);
                 break;
 
             default:
@@ -138,11 +136,15 @@ gdm_display_access_file_class_init (GdmDisplayAccessFileClass *access_file_class
                                           NULL,
                                           G_PARAM_READABLE);
         g_object_class_install_property (object_class, PROP_PATH, param_spec);
+        g_type_class_add_private (access_file_class, sizeof (GdmDisplayAccessFilePrivate));
 }
 
 static void
 gdm_display_access_file_init (GdmDisplayAccessFile *access_file)
 {
+        access_file->priv = G_TYPE_INSTANCE_GET_PRIVATE (access_file,
+                                                         GDM_TYPE_DISPLAY_ACCESS_FILE,
+                                                         GdmDisplayAccessFilePrivate);
 }
 
 static void
@@ -154,14 +156,14 @@ gdm_display_access_file_finalize (GObject *object)
         file = GDM_DISPLAY_ACCESS_FILE (object);
         parent_class = G_OBJECT_CLASS (gdm_display_access_file_parent_class);
 
-        if (file->fp != NULL) {
+        if (file->priv->fp != NULL) {
             gdm_display_access_file_close (file);
         }
-        g_assert (file->path == NULL);
+        g_assert (file->priv->path == NULL);
 
-        if (file->username != NULL) {
-                g_free (file->username);
-                file->username = NULL;
+        if (file->priv->username != NULL) {
+                g_free (file->priv->username);
+                file->priv->username = NULL;
                 g_object_notify (object, "username");
         }
 
@@ -295,7 +297,7 @@ _create_xauth_file_for_user (const char  *username,
                 g_set_error (error,
                              GDM_DISPLAY_ERROR,
                              GDM_DISPLAY_ERROR_GETTING_USER_INFO,
-                             _("could not find user “%s” on system"),
+                             _("could not find user \"%s\" on system"),
                              username);
                 goto out;
 
@@ -308,7 +310,7 @@ _create_xauth_file_for_user (const char  *username,
         g_debug ("GdmDisplayAccessFile: creating xauth directory %s", template);
         /* Initially create with mode 01700 then later chmod after we create database */
         errno = 0;
-        dir_name = g_mkdtemp (template);
+        dir_name = gdm_make_temp_dir (template);
         if (dir_name == NULL) {
                 g_set_error (error,
                              G_FILE_ERROR,
@@ -403,15 +405,15 @@ gdm_display_access_file_open (GdmDisplayAccessFile  *file,
         GError *create_error;
 
         g_return_val_if_fail (file != NULL, FALSE);
-        g_return_val_if_fail (file->fp == NULL, FALSE);
-        g_return_val_if_fail (file->path == NULL, FALSE);
+        g_return_val_if_fail (file->priv->fp == NULL, FALSE);
+        g_return_val_if_fail (file->priv->path == NULL, FALSE);
 
         create_error = NULL;
-        file->fp = _create_xauth_file_for_user (file->username,
-                                                &file->path,
-                                                &create_error);
+        file->priv->fp = _create_xauth_file_for_user (file->priv->username,
+                                                      &file->priv->path,
+                                                      &create_error);
 
-        if (file->fp == NULL) {
+        if (file->priv->fp == NULL) {
                 g_propagate_error (error, create_error);
                 return FALSE;
         }
@@ -436,11 +438,6 @@ _get_auth_info_for_display (GdmDisplayAccessFile *file,
         gdm_display_is_local (display, &is_local, NULL);
 
         if (is_local) {
-                /* We could just use FamilyWild here except xauth
-                 * (and by extension su and ssh) doesn't support it yet
-                 *
-                 * https://bugs.freedesktop.org/show_bug.cgi?id=43425
-                 */
                 char localhost[HOST_NAME_MAX + 1] = "";
                 *family = FamilyLocal;
                 if (gethostname (localhost, HOST_NAME_MAX) == 0) {
@@ -473,7 +470,7 @@ gdm_display_access_file_add_display (GdmDisplayAccessFile  *file,
         gboolean display_added;
 
         g_return_val_if_fail (file != NULL, FALSE);
-        g_return_val_if_fail (file->path != NULL, FALSE);
+        g_return_val_if_fail (file->priv->path != NULL, FALSE);
         g_return_val_if_fail (cookie != NULL, FALSE);
 
         add_error = NULL;
@@ -512,7 +509,7 @@ gdm_display_access_file_add_display_with_cookie (GdmDisplayAccessFile  *file,
         gboolean display_added;
 
         g_return_val_if_fail (file != NULL, FALSE);
-        g_return_val_if_fail (file->path != NULL, FALSE);
+        g_return_val_if_fail (file->priv->path != NULL, FALSE);
         g_return_val_if_fail (cookie != NULL, FALSE);
 
         _get_auth_info_for_display (file, display,
@@ -530,8 +527,8 @@ gdm_display_access_file_add_display_with_cookie (GdmDisplayAccessFile  *file,
         /* FIXME: We should lock the file in case the X server is
          * trying to use it, too.
          */
-        if (!XauWriteAuth (file->fp, &auth_entry)
-            || fflush (file->fp) == EOF) {
+        if (!XauWriteAuth (file->priv->fp, &auth_entry)
+            || fflush (file->priv->fp) == EOF) {
                 g_set_error (error,
                         G_FILE_ERROR,
                         g_file_error_from_errno (errno),
@@ -541,18 +538,6 @@ gdm_display_access_file_add_display_with_cookie (GdmDisplayAccessFile  *file,
                 display_added = TRUE;
         }
 
-        /* If we wrote a FamilyLocal entry, we still want a FamilyWild
-         * entry, because it's more resiliant against hostname changes
-         *
-         */
-        if (auth_entry.family == FamilyLocal) {
-                auth_entry.family = FamilyWild;
-
-                if (XauWriteAuth (file->fp, &auth_entry)
-                    && fflush (file->fp) != EOF) {
-                        display_added = TRUE;
-                }
-        }
 
         g_free (auth_entry.address);
         g_free (auth_entry.number);
@@ -561,26 +546,85 @@ gdm_display_access_file_add_display_with_cookie (GdmDisplayAccessFile  *file,
         return display_added;
 }
 
+gboolean
+gdm_display_access_file_remove_display (GdmDisplayAccessFile  *file,
+                                        GdmDisplay            *display,
+                                        GError               **error)
+{
+        Xauth           *auth_entry;
+        unsigned short  family;
+        unsigned short  address_length;
+        char           *address;
+        unsigned short  number_length;
+        char           *number;
+        unsigned short  name_length;
+        char           *name;
+
+
+        g_return_val_if_fail (file != NULL, FALSE);
+        g_return_val_if_fail (file->priv->path != NULL, FALSE);
+
+        _get_auth_info_for_display (file, display,
+                                    &family,
+                                    &address_length,
+                                    &address,
+                                    &number_length,
+                                    &number,
+                                    &name_length,
+                                    &name);
+
+        auth_entry = XauGetAuthByAddr (family,
+                                       address_length,
+                                       address,
+                                       number_length,
+                                       number,
+                                       name_length,
+                                       name);
+        g_free (address);
+        g_free (number);
+        g_free (name);
+
+        if (auth_entry == NULL) {
+                g_set_error (error,
+                             GDM_DISPLAY_ACCESS_FILE_ERROR,
+                             GDM_DISPLAY_ACCESS_FILE_ERROR_FINDING_AUTH_ENTRY,
+                             "could not find authorization entry");
+                return FALSE;
+        }
+
+        XauDisposeAuth (auth_entry);
+
+        if (fflush (file->priv->fp) == EOF) {
+                g_set_error (error,
+                             G_FILE_ERROR,
+                             g_file_error_from_errno (errno),
+                             "%s", g_strerror (errno));
+                return FALSE;
+        }
+
+        return TRUE;
+}
+
 void
 gdm_display_access_file_close (GdmDisplayAccessFile  *file)
 {
         char *auth_dir;
 
         g_return_if_fail (file != NULL);
-        g_return_if_fail (file->fp != NULL);
-        g_return_if_fail (file->path != NULL);
+        g_return_if_fail (file->priv->fp != NULL);
+        g_return_if_fail (file->priv->path != NULL);
 
         errno = 0;
-        if (g_unlink (file->path) != 0) {
+        if (g_unlink (file->priv->path) != 0) {
                 g_warning ("GdmDisplayAccessFile: Unable to remove X11 authority database '%s': %s",
-                           file->path,
+                           file->priv->path,
                            g_strerror (errno));
         }
 
         /* still try to remove dir even if file remove failed,
            may have already been removed by someone else */
         /* we own the parent directory too */
-        auth_dir = g_path_get_dirname (file->path);
+        auth_dir = g_path_get_dirname (file->priv->path);
         if (auth_dir != NULL) {
                 errno = 0;
                 if (g_rmdir (auth_dir) != 0) {
@@ -591,16 +635,16 @@ gdm_display_access_file_close (GdmDisplayAccessFile  *file)
                 g_free (auth_dir);
         }
 
-        g_free (file->path);
-        file->path = NULL;
+        g_free (file->priv->path);
+        file->priv->path = NULL;
         g_object_notify (G_OBJECT (file), "path");
 
-        fclose (file->fp);
-        file->fp = NULL;
+        fclose (file->priv->fp);
+        file->priv->fp = NULL;
 }
 
 char *
 gdm_display_access_file_get_path (GdmDisplayAccessFile *access_file)
 {
-        return g_strdup (access_file->path);
+        return g_strdup (access_file->priv->path);
 }
